package com.websphere.examples;

import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.Connection;
import javax.jms.JMSException;

import com.ibm.msg.client.wmq.WMQConstants;

import com.ibm.msg.client.jms.JmsFactoryFactory;
import com.ibm.msg.client.jms.JmsConnectionFactory;

public abstract class JmsBase
{
	private String   host              = "localhost"         ;
	private int      port              = 1414                ;
	private String   channel           = "SYSTEM.DEF.SVRCONN";
	private String   queueManagerName  = "host1/qm1"         ;
	private String   queueName         = "queue1"            ;  
	private boolean  clientTransport   = false               ;

	protected Session      session     = null;
    protected Queue        destination = null;
	
	protected Connection   connection  = null;
                         // статус выполнения приложения
	protected static int status = 1; 

	private void createConnection()
	{
		JmsFactoryFactory    ff;
		JmsConnectionFactory cf;
	    try {
	    	ff = JmsFactoryFactory.getInstance(WMQConstants.WMQ_PROVIDER);
	    	cf = ff.createConnectionFactory();

	    	// Set the properties
	    	cf.setStringProperty(WMQConstants.WMQ_HOST_NAME, host   );
	    	cf.setIntProperty   (WMQConstants.WMQ_PORT     , port   );
	    	cf.setStringProperty(WMQConstants.WMQ_CHANNEL  , channel);
	    	if (clientTransport) {
	    		cf.setIntProperty(WMQConstants.WMQ_CONNECTION_MODE, WMQConstants.WMQ_CM_CLIENT);
	    	} else {
	    		cf.setIntProperty(WMQConstants.WMQ_CONNECTION_MODE, WMQConstants.WMQ_CM_BINDINGS);
	    	}
	    	cf.setStringProperty(WMQConstants.WMQ_QUEUE_MANAGER, queueManagerName);
	    	
	    	connection  = cf.createConnection();
	    } catch (JMSException jmsex) {
	    	recordFailure(jmsex);
	    }
	}
	/**
	 * Record this run as failure.
	 * @param ex исключение
	 */
	protected void recordFailure(Exception ex)
	{
		if (ex != null) {
			if (ex instanceof JMSException) {
				processJMSException((JMSException) ex);
			} else {
				System.err.println(ex);
			}
		}
		System.err.println("FAILURE");
		status = -1;
		return;
	}

	/**
	 * Обработка JMSException и внутренних ошибок
	 * @param jmse Exception
	 */
	protected void processJMSException(JMSException jmse)
	{
		System.err.println(jmse);
		Throwable innerException = jmse.getLinkedException();
		if (innerException != null) {
			System.err.println("Inner exception :");
		}
		while (innerException != null) {
			System.err.println(innerException);
			innerException = innerException.getCause();
		}
		return;
	}

	protected abstract void doAction();
	
	public JmsBase()
	{
		try {
		    try {
		    	createConnection();
		    	session     = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
		    	destination = session.createQueue(queueName);
		        // Старт подключения
		    	connection.start();
		    	// Выполнение действий
		    	doAction();
		    }
		    catch (JMSException jmsex) {
		      recordFailure(jmsex);
		    } 
		} finally {
	    	try {
	    		if (session != null)
	    			session.close();
	    		if (connection != null)
	    			connection.close();
	    	} catch (JMSException jmsex) {
	    		System.err.println("" + jmsex.getMessage());
	    		recordFailure(jmsex);
	    	}
	    }
	}
}
